#!/usr/bin/env python

""" hitran

 usage:
 hitran [options] line_parameter_database

 command line options:
   -h  help
   -i  int       isotope number  (default: all)
   -m  int       molecule number (default: all)
   -o  string    output file (default: standard output)
   -S  float     minimum line strength to accept (default: S=0.0)
   -x  interval  lower and upper end of wavenumber range (comma separated pair without blanks)
    
 Note: at least wavenumber range or molecule has to be selected!
"""
##############################################################################################################
#####  ToDo:                                                                                             #####
#####                                                                                                    #####
##############################################################################################################

import os
import sys
from string import *
from types  import *
from time   import clock

from pairTypes import Interval

##########################################################################################################################################
def extract_hitran (file, xLimits, Mol, Iso=0, Smin=0.0):
	""" Read Hitran formatted database. """
	try:
		hitran = open (file)
	except IOError:
		raise SystemExit, 'opening Hitran data file "' + file + '" failed!'
	# wavenumber interval to be searched
	xBegin, xHigh = xLimits.limits()
	# initialize time and search first useful line
	time0  = clock()
 	if Mol<=0:
	 	lines = extract_All (hitran, xBegin,xHigh)
 	elif not (Iso or Smin):
		print 'extract mol'
	 	lines = extract_Mol (hitran, xBegin,xHigh, Mol)
 	elif Iso==0 and Smin>0.0:
	 	lines = extract_MolStr (hitran, xBegin,xHigh, Mol, Smin)
 	elif Smin<=0.0:
	 	lines = extract_MolIso (hitran, xBegin,xHigh, Mol, Iso)
 	else:
	 	lines = extract_MolIsoStr (hitran, xBegin,xHigh, Mol, Iso, Smin)
	hitran.close()
	return lines
	#return molecules, isotopes, positions, strengths, energies, airWidths, selfWidths, tempExps
##########################################################################################################################################
def bisect_first_line (db, xBegin, xEnd, iw=3,lw=15):
 	""" Search and return first line of Hitran or Geisa formatted database. """
	if iw>2:
		# hitran database, skip header section with comment lines (indicated with mol=00)
		mol  = -1
		while mol<1:  record=db.readline(); mol = int(record[:2])
	else:
		# hitran database, skip header section with comment lines (indicated with mol=00)
		record=db.readline()
	# first record in file
	lenRec   = len(record)
	locFirst = db.tell() - lenRec
	recFirst = record
	xFirst   = float(recFirst[iw:lw])
	# move to last record in file 348043
	db.seek(-lenRec,2)
	locLast = db.tell()
	recLast = db.readline()
	xLast   = float(recLast[iw:lw])
	# check spectral range
	if  xFirst <= xBegin < xEnd <= xLast:
		pass
	elif  xFirst > xEnd or xLast < xBegin:
	 	raise SystemExit, 'requested spectral range not in database ' + str(xFirst) + ' --- ' + str(xLast)
	else:
	 	print 'requested spectral range only partly in database ' + str(xFirst) + ' --- ' + str(xLast)
	# record number of very first and last lines (locFirst, locLast are byte numbers)
	lineFirst = locFirst/lenRec
	lineLast  = (locLast-locFirst)/lenRec
	#print locFirst, lineFirst, 'very first', recFirst, '\n', locLast, lineLast, 'very last ', recLast[:-1]
	# bisecting to desired first line
	while lineLast-lineFirst > 1:
		mid=(lineLast+lineFirst)/2
		db.seek(locFirst+mid*lenRec)
		recMid=db.readline()
		xMid = float(recMid[iw:lw])
		# print 3*'%8i ' % (lineFirst, mid, lineLast), 3*' %12.6f' % (xFirst, xMid, xLast)
		if   xBegin<xMid: lineLast =mid; xLast=xMid
		elif xBegin>xMid: lineFirst=mid; xFirst=xMid
		else:
			# backstep: there are possibly several lines at exactly this position
			while 1:
				db.seek(-2*lenRec,1); rec=db.readline(); xMid=float(rec[iw:lw])
				if xMid<xBegin: return        db.readline()
	
	if    xMid<xBegin:      record = db.readline()
	else:                   record = recMid
 	print "# first accepted line at record number", mid, "found in", clock(), "sec", '  Wavenumber:', record[iw:lw], ' molID:', record[:2]
	return record

##########################################################################################################################################
def extract_All (hitran, xBegin, xHigh):
	""" Read all lines up to an upper wavenumber limit from Hitran formatted database. """
	# proceed to first requested line
	record = bisect_first_line (hitran, xBegin, xHigh)
	# initialize list if lines
	lines = []
	# collect lines
	while record:
		mol    = int(record[:2])
		if mol>0:
			wvn = float(record[3:15])
			if wvn<=xHigh:  lines.append(record)
			else:           break
		# read next record
		record = hitran.readline()

	print 'first line not accepted \n', record
	return lines
##########################################################################################################################################
def extract_Mol (hitran, xBegin,xHigh, GetMol):
	""" Read lines of a given molecule up to an upper wavenumber limit from Hitran formatted database. """
	# proceed to first requested line
	record = bisect_first_line (hitran, xBegin, xHigh)
	# initialize list if lines
	lines = []
	# collect lines
	while record:
		mol = int(record[:2])
		wvn = float(record[3:15])
		if wvn>xHigh: break
		if mol==GetMol: lines.append(record)
		# read next record
		record = hitran.readline()

	print 'first line not accepted \n', record
	return lines
##########################################################################################################################################
def extract_MolIso (hitran, xBegin,xHigh, GetMol, GetIso):
	""" Read lines of a given molecule/isotope up to an upper wavenumber limit from Hitran formatted database. """
	# proceed to first requested line
	record = bisect_first_line (hitran, xBegin, xHigh)
	# initialize list if lines
	lines = []
	# collect lines
	while record:
		mol = int(record[:2])
		iso = int(record[2:3])
		wvn = float(record[3:15])
		if wvn>xHigh: break
		if mol==GetMol and iso==GetIso: lines.append(record)
		# read next record
		record = hitran.readline()

	print 'first line not accepted \n', record
	return lines

##########################################################################################################################################
def extract_MolStr (hitran, xBegin,xHigh, GetMol, StrMin):
	""" Read strong lines of a given molecule up to an upper wavenumber limit from Hitran formatted database. """
	# proceed to first requested line
	record = bisect_first_line (hitran, xBegin, xHigh)
	# initialize list if lines
	lines = []
	# collect lines
	while record:
		mol = int(record[:2])
		wvn = float(record[3:15])
		str = float(record[15:25])
		if wvn>xHigh: break
		if mol==GetMol and str>=StrMin: lines.append(record)
		# read next record
		record = hitran.readline()

	print 'first line not accepted \n', record
	return lines

##########################################################################################################################################
def extract_MolIsoStr (hitran, xBegin,xHigh, GetMol, GetIso, StrMin):
	""" Read strong lines of a given molecule/isotope up to an upper wavenumber limit from Hitran formatted database. """
	# proceed to first requested line
	record = bisect_first_line (hitran, xBegin, xHigh)
	# initialize list if lines
	lines = []
	# collect lines
	while record:
		mol = int(record[:2])
		iso = int(record[2:3])
		str = float(record[15:25])
		if wvn>xHigh: break
		if mol==GetMol and iso==GetIso and str>=StrMin:  lines.append(record)
		# read next record
		record = hitran.readline()
	print 'first line not accepted \n', record
	return lines
##########################################################################################################################################

if (__name__ == "__main__"):
	from io import open_outFile

	from command_parser import parse_command, standardOptions
        opts = standardOptions + [  # h=help, c=commentChar, o=outFile
	       {'ID': 'i', 'name': 'IsoNr',  'type': IntType, 'default': 0},
               {'ID': 'm', 'name': 'MolNr', 'type': IntType, 'default': 0},
               {'ID': 'S', 'name': 'Smin', 'type': FloatType,   'default': 0.0, 'constraint': 'Smin>=0.0'},
	       {'ID': 'x', 'name': 'xLimits', 'type': Interval, 'default': Interval(0.0,99999.9), 'constraint': 'xLimits.lower>=0.0'}
               ]

	files, options, commentChar, outFile = parse_command (opts, 1)

	for opt in opts:
		if opt.has_key('name') and opt.has_key('type'):  exec opt['name'] + ' = ' + repr(options.get(opt['name']))
 	
    	if options.has_key('h'):
    		print __doc__%globals();  raise SystemExit, " end of hitran help"
	elif MolNr or xLimits.size()<50000.0:
		# Read lines from HITRAN line parameter file
		lines = extract_hitran (files[0], xLimits, MolNr, IsoNr, Smin)
		print len(lines), ' lines'
	else:
		# at least molecule or wavenumber range needed
		raise SystemExit, ' ERROR: neither molecule nor wavenumber range specified!'

	# open an output file if explicitely specified (otherwise use standard output)
	out = open_outFile (outFile, commentChar='00')
	# print extracted lines
	for line in lines: out.write (line)
	# close the output file (if its not stdout)
	if outFile: out.close()
